//-----------------------------------------------------------------------------
//
// A bare-bones skeleton based on the Microsoft DirectX SDK Tutorial 1
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Header files
//-----------------------------------------------------------------------------

#include "core.h"

//----------------------------------------------------------------------------
// Global variables
//----------------------------------------------------------------------------

LPDIRECT3DTEXTURE8 why;
int red,green,blue,xshift,yshift;

//----------------------------------------------------------------------------
// InitD3D() : Initializes Direct3D
//----------------------------------------------------------------------------

HRESULT Init( HWND hWnd )
{

	InitD3D(hWnd);

	// Seting up data
	why = LoadTexture("test.bmp");
	red = 0;
	green = 100;
	blue = 255;
	xshift = 0;
	yshift = 0;
	return S_OK;
}

//----------------------------------------------------------------------------
// DrawScene() : Draws the contents of the scene
//----------------------------------------------------------------------------
void DrawScene()
{
	// Drawing code goes here...
	RECT l1,l2;

	l1.top = 0+yshift;
	l1.left = 0+xshift;
	l1.right = 208+xshift;
	l1.bottom = 263+yshift;
	l2.top = 0;
	l2.left = 0;
	l2.right = 208;
	l2.bottom = 263;
	BltSpriteEx(&l1,why,&l2,0,D3DCOLOR_XRGB(red,green,blue),0,NULL);
	return;
}

//----------------------------------------------------------------------------
// UpdateStuff() : Updates changes in the world
//----------------------------------------------------------------------------
void UpdateStuff() {
	static int r=0,g=0,b=0,xd=0,yd=0;

	//Rotates Red
	if (r==0){red-=2;if (red <= 0){red = 0;r=1;}}
	else{red+=2;if (red >= 255){ red = 255;	r=0;}}

	//Rotates Green
	if (g==0){green--;if (green <= 0){green = 0;g=1;}}
	else{green++;if (green >= 255){green = 255;g=0;}}

	//Rotates Blue
	if (b==0){blue-=3;if (blue <= 0){blue = 0;b=1;}}
	else{blue+=3;if (blue >= 255){blue = 255;b=0;}}

	if (xd==0){xshift-=1;if (xshift <= 0){xshift = 0;xd=1;}}
	else{xshift+=1;if (xshift >= 432){xshift = 432;xd=0;}}

	if (yd==0){yshift-=1;if (yshift <= 0){yshift = 0;yd=1;}}
	else{yshift+=1;if (yshift >= 217){yshift = 217;yd=0;}}
}
//----------------------------------------------------------------------------
// Render() : Prepares for drawing the scene
//----------------------------------------------------------------------------
void Render()
{
	if( dx8.pd3dDevice == NULL)
		return;
	
	// Clear the backbuffer to a black color
	dx8.pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER , 
		D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );
	
	// Begin the scene
	dx8.pd3dDevice->BeginScene();
	
	// Rendering of scene objects
	DrawScene();
	
	// End the scene
	dx8.pd3dDevice->EndScene();
	
	// Present the backbuffer contents to the display
	dx8.pd3dDevice->Present( NULL, NULL, NULL, NULL );
}


//----------------------------------------------------------------------------
// MsgProc() : The window's message handler
//----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg ){
		case WM_KEYUP: 
			switch (wParam) { 
				case VK_ESCAPE:
					PostQuitMessage( 0 );
                    break;
				case VK_UP:
					break;
				case VK_DOWN:
					break;
				case VK_LEFT:
					break;
				case VK_RIGHT:
					break;

            } 
            return 0;
		case WM_CHAR:
			return 0;
		case WM_DESTROY: // If you use Windowed mode process loss of it
			PostQuitMessage( 0 );
			return 0;
		
		case WM_PAINT:
			// NOTE: In a full-screen app, you may choose to draw the screen at
			// a different point, such as using a timer message or outside of
			// the windows message loop
	        Render();
			ValidateRect( hWnd, NULL );
			return 0;
	}
	return DefWindowProc( hWnd, msg, wParam, lParam );
}


//----------------------------------------------------------------------------
// WinMain() : The application's entry point
//----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
	MSG mssg;                // message from queue
	LONGLONG cur_time;       // current time
	DWORD  time_count=1;    // ms per frame, default if no performance counter
	LONGLONG perf_cnt;       // performance timer frequency
	LONGLONG next_time=0;    // time to render next frame
	BOOL move_flag=TRUE;     // flag noting if we have moved yet

	// Register the window class
	WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_HREDRAW | CS_VREDRAW, MsgProc, 0, 
		sizeof(DWORD), GetModuleHandle(NULL), NULL, LoadCursor(NULL, IDC_ARROW), 
		(HBRUSH) GetStockObject(BLACK_BRUSH), NULL, "Tetris Extreme", NULL };
	RegisterClassEx( &wc );
	
	// Create the application's window
	HWND hWnd = CreateWindow( "Tetris Extreme", "Tetris Extreme", 
		WS_POPUP, 100, 100, 640, 480,
		GetDesktopWindow(), NULL, wc.hInstance, NULL );
	
	// Initialize Direct3D
	if( SUCCEEDED( Init( hWnd ) ) )
	{
		// Show the window
		ShowWindow( hWnd, SW_SHOWDEFAULT );
		UpdateWindow( hWnd ); // Forces a WM_PAINT
		
		if (QueryPerformanceFrequency((LARGE_INTEGER *) &perf_cnt)) {
			// yes, set time_count and timer choice flag
			time_count=perf_cnt/25;  // calculate time per frame based on frequency
			QueryPerformanceCounter((LARGE_INTEGER *) &next_time);
		} else {
			return FALSE;
		}

		// prime the message structure
		PeekMessage( &mssg, NULL, 0, 0, PM_NOREMOVE);
		// run till completed
		while (mssg.message!=WM_QUIT) {
			if (PeekMessage( &mssg, NULL, 0, 0, PM_REMOVE)) {
				TranslateMessage(&mssg);
				DispatchMessage(&mssg);
			} else {
				if (move_flag) { //update world
					UpdateStuff();
					move_flag=FALSE;
				}
				else{
					QueryPerformanceCounter((LARGE_INTEGER *) &cur_time);
					if (cur_time>next_time) { //Is it time to draw
						Render();
						next_time += time_count;
						if (next_time < cur_time)
							next_time = cur_time + time_count;
						move_flag=TRUE;
					}
				}
			}
		}
	}
	
	// Clean up everything and exit the app
	Cleanup();
	UnregisterClass( "Tetris Extreme", wc.hInstance );
	return 0;
}

